/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt
 *
 * de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg003HeuschBoesefeldt wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion.nw.vrz;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.Eak;
import de.bsvrz.kex.tls.osi7.IllegalOsi7TelegramStructureException;
import de.bsvrz.kex.tls.osi7.TlsApplicationLayer;
import de.bsvrz.kex.tls.osi7.TlsData;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.ArrayList;
import java.util.List;

/**
 * Umsetungsmodul für die FG 3, Standard (alle Typen der FG 3 gemäß TLS 2002). Angemeldet werden aber nur die
 * Telegrammtypen, die bei allen FG 3 DE identisch sind, also DeFehler, Kanalsteuerung etc. Umsetzungsmodul enthält zwar
 * alle Umsetzungsmethoden, für einen konkreten FG 3 Typ (z.B. Helligkeit) ist aber für dieses De in der Konfiguration
 * als Umsetzungsklasse aber die spezielle, von dieser Klasse abgeleitete Klasse (bei Helligkeit {@link
 * de.bsvrz.kex.tls.osi7.conversion.Fg003Typ061} ) zu verwenden, in der dann die spezifischen Umsetzungsmethoden
 * angemeldet werden. Bei FG 3 De ist Angabe der speziellen Umsetzungsklasse in der Konfiguration obligatorisch, es wird
 * keine Umsetzungklasse bei Fehlen einer solchen Angabe standardmäßig verwendet!
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class Fg003HeuschBoesefeldt extends FgAllHeuschBoesefeldt {
	// ------------------------------------------------------------------------
	// VARIABLENDEKLARATIONEN
	// ------------------------------------------------------------------------

	// Sonstige ---------------------------------------------------------------
	/** DebugLogger für Debug-Ausgaben */
	private static final Debug debug = Debug.getLogger();
	// Private ----------------------------------------------------------------
	/**
	 * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
	 * Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2", "typ.deUfd"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Empfangsanmeldung als Empfänger.</li> <li> In der Methode {@link #update} wird für jede zur
	 * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag
	 * wie folgt gebildet wird (hier für den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
	 * Dabei enthält <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Für jede
	 * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere passende Eintrage
	 * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere ATG/ASP-Kombinationen in
	 * einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines Fachmodells):</li>
	 * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
	 */
	private final Object[][] davToTls = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
			{"atg.tlsGloDeBlockAbruf", "asp.tlsAbruf", "255", "255", "255", "typ.deUfd"}, {"atg.tlsGloDeBlockSenden", "asp.tlsVorgabe", "255", "255", "255", "typ.deUfd"}, {"atg.tlsGloKanalSteuerung", "asp.parameterSoll", "255", "29", "2", "typ.deUfd"}, {"atg.tlsGloGeographischeKenndaten", "asp.tlsVorgabe", "255", "36", "3", "typ.deUfd"}, {"atg.tlsUfdBetriebsParameter", "asp.parameterSoll", "3", "32", "3", "typ.deUfd"}};

	/**
	 * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat folgenden
	 * Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deUfd"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird für jede zum De-Block passende
	 * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für
	 * den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
	 * DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
	 * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
	 * De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende {@link DataDescription} für die
	 * zu sendenden Daten.</li> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine Sendeanmeldung als Quelle.
	 * Sind mehrere passende Eintrage vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch kann ein
	 * De-Block mehrfach ausgewert und an verschiedene Attributgruppen weitergerleitet werden (z.B. protokollnahe
	 * TLS-Attributgruppe und Attributgruppe eine Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer
	 * "Wildcard" (passt immer). </ul>
	 */
	private final Object[][] tlsToDav = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
			{"atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.deUfd"}, {"atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.deUfd"}, {"atg.tlsUfdErgänzendeDeFehlerMeldung", "asp.tlsAntwort", "3", "14", "1", "typ.deUfd"}, {"atg.tlsUfdNegativeQuittung", "asp.tlsAntwort", "3", "16", "2", "typ.deUfd"}, {"atg.tlsUfdBetriebsParameter", "asp.parameterIst", "3", "32", "3", "typ.deUfd"}

	};

	// ------------------------------------------------------------------------
	// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
	// ------------------------------------------------------------------------

	/**
	 * Erzeugt ein Objekt vom Typ Fg003Default
	 *
	 * @param daf Datenverteilerinterface.
	 * @param eak Eak des DE.
	 * @param de  Das DE.
	 */
	public Fg003HeuschBoesefeldt(ClientDavInterface daf, Eak eak, De de) {
		super(daf, eak, de);
		subscribeAllData(tlsToDav, davToTls);
	}

	// ------------------------------------------------------------------------
	// SONSTIGE METHODEN
	// ------------------------------------------------------------------------

	/**
	 * FG 3 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Ergänzt globale Behandlung De-Fehler in {@link
	 * #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}. Beim Empfang einer De-Gutmeldung wird die De initialisiert, indem
	 * <ul> <li>Ufd-kanalsteuerung {@link #Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll}</li>
	 * <li>Ufd-betriebsparameter{@link #Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterSoll}</li> </ul> versendet
	 * werden.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

		IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();
		if (DEFehlerStatus != null) {
			debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());
			if (DEFehlerStatus.getName().equals("ok")) {
				// Bei der Kommunikation mit der VRZ werden keine Daten bei De=ok versendet.
				// sendInitialDeBlock(deBlock.getKey(), 29, 2); // UFD-Kanalsteuerung senden
				// sendInitialDeBlock(deBlock.getKey(), 32, 3); // UFD-Betriebsparameter senden
			}
			else {
				List deList = new ArrayList();
				deList.add(_de);
				TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
			}
		}
		return resultData;
	}

	public DeBlock Fg003Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		String deFehlerStatus;
		String deProjektierungsStatus;
		String deKanalStatus;
		int herstellerDefinierterCode;
		int herstellerCode;

		// Umsetzung
		Data data = result.getData();// Datensatz holen
		deFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getText();
		deProjektierungsStatus = data.getScaledValue("DEProjektierungsStatus").getText();
		deKanalStatus = data.getUnscaledValue("DEKanalStatus").getText();
		herstellerDefinierterCode = data.getUnscaledValue("HerstellerDefinierterCode").intValue();
		herstellerCode = data.getUnscaledValue("Hersteller").intValue();

		// DeBlock als TlsData aufbauen...
		int fehlerCode = 0;

		if (deFehlerStatus.equals("ok")) fehlerCode = 0;
		if (deFehlerStatus.equals("StörEAK")) fehlerCode = 1;
		if (deFehlerStatus.equals("StörSM")) fehlerCode = 2;

		if (deProjektierungsStatus.equals("Projektierung Fehler")) fehlerCode += 4;
		if (deKanalStatus.equals("passiv")) fehlerCode += 8;
		fehlerCode += herstellerDefinierterCode * 16;

		TlsData deApplicationData;
		deApplicationData = new TlsData(2);
		deApplicationData.writeIntAsByte(fehlerCode);
		deApplicationData.writeIntAsByte(herstellerCode);

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 1, 1, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG 3 : Typ: 14 Id: 1 Name: Ergänzende DE-Fehlermeldung (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          [ 9..29 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1..254 ]
	 * Byte 3           Typ der DE-Daten                        [ 14 ]
	 * Byte 4           Herstellercode                          (s. Anh. 7, Kap. 2)
	 * Byte 5           Anzahl TLS  Fehlerbytes                [ 4 ]
	 * Byte 6           TLS - Fehlerbyte 1                      Codierung siehe unten
	 * Byte 7           TLS - Fehlerbyte 2                      Codierung siehe unten
	 * Byte 8           TLS - Fehlerbyte 3                      Codierung siehe unten
	 * Byte 9           TLS - Fehlerbyte 4                      Codierung siehe unten
	 * Byte 10          Anzahl Hersteller Fehlerbytes           [ 0..20 ]
	 * Byte 11          Hersteller-Fehlerbyte 1                 herstellerspez. Codierung
	 * Byte..           Hersteller-Fehlerbyte n
	 * </pre>
	 * Inhalt TLS-Fehlerbytes:<br> Die Fehlerbytes sind bitweise codiert. Jedes Bit definiert einen bestimmten Fehler,
	 * wobei das Bit gleich 1 gesetzt ist.<br> Es können auch mehrere Bit gleichzeitig gesetzt werden. Der Inhalt der
	 * Fehlerbytes ist optional. D.h. bei einer Realisierung dieses Telegrammtyps müssen vom Hersteller nicht alle in der
	 * Tabelle definierten Fehlercodes erfüllt werden. Für spätere Definitionen reservierte Bits bzw. nicht verwendete Bits
	 * sind gleich Null zu setzen.
	 * <pre>
	 *         Bit 7      Bit 6       Bit 5       Bit 4          Bit 3         Bit 2       Bit 1       Bit 0
	 * Byte 1  Reserve    Reserve     RNR am      Lokalbus       Reserve       Reserve     Reserve     Teilstörung
	 *                                Lokalbus1   ausgefallen
	 * Byte 2  Reserve    Reserve     Reserve     Reserve        Rohwert       Sensor      RNR am      Subbus
	 *                                                           außerhalb     defekt      Subbus      ausgefallen
	 *                                                           der Grenzen
	 * Byte 3  Reserve    Reserve     Reserve     Reserve        Reserve       Reserve     Reserve     Reserve
	 * Byte 4  Reserve    Reserve     Reserve     Reserve        Reserve       Reserve     Reserve     Reserve
	 * </pre>
	 * 1) nur in Verbindung mit Stö/SM=1 im Byte 4 der DE-Block-Struktur im Typ 1<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ014Id001AtgTlsUfdErgänzendeDeFehlerMeldungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		// Fehler-Bytes aus Telegramm -----------------------------------------------------------------------------------
		deBlock.getDeData().readUnsignedByteAsInt();// Anzahl der folgenden Bytes überlesen, da immer == 4 !
		int _TlsFehler1_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _TlsFehler2_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _TlsFehler3_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _TlsFehler4_ = deBlock.getDeData().readUnsignedByteAsInt();

		Data.NumberArray _TlsFehler_ = data.getItem("FehlercodesTLS").getUnscaledArray("Fehler");
		_TlsFehler_.setLength(4);
		_TlsFehler_.getValue(0).set(_TlsFehler1_);
		_TlsFehler_.getValue(1).set(_TlsFehler2_);
		_TlsFehler_.getValue(2).set(_TlsFehler3_);
		_TlsFehler_.getValue(3).set(_TlsFehler4_);

		// Hersteller Fehler-Bytes aus Telegramm -----------------------------------------------------------------------
		int count = deBlock.getDeData().readUnsignedByteAsInt();
		int[] _HerstellerFehler_ = new int[count];
		for (int i = 0; i < count; i++) {
			_HerstellerFehler_[i] = deBlock.getDeData().readUnsignedByteAsInt();
		}

		Data.NumberArray _Fehler_ = data.getItem("FehlercodesHersteller").getUnscaledArray("Fehler");
		_Fehler_.setLength(count);
		for (int i = 0; i < count; i++) {
			_Fehler_.getValue(i).set(_HerstellerFehler_[i]);
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 16 Id: 2 Name: Negative Quittung (Antwort) TLS-Datenbeschreibung:<br>
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 4 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1..254 ]
	 * Byte 3           Typ der DE-Daten                        [ 16 ]
	 * Byte 4           Fehlerursache                           siehe unten
	 * Byte 5           Herstellercode                          siehe Liste in Anhang 7, Kap. 2
	 * </pre>
	 * Inhalt Byte: Fehlerursache unter FG 3:
	 * <pre>
	 *        0:   sonstige Fehlerursache
	 *        1:   unbekannte oder nicht auswertbare ID
	 *        2:   unbekannter oder nicht auswertbarer Typ
	 *
	 *   3..127:   reserviert für spätere Definitionen
	 * 128..255:   frei für herstellerdefinierte Ursachencodes
	 *
	 * </pre>
	 * Inhalt Byte Herstellercode<br> Der Herstellercode ist ein für jeden Hersteller eindeutig zugeordneter Code, der der
	 * Zentrale die Zuordnung der Nachrichten erleichtern soll.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */

	public ResultData Fg003Typ016Id002AtgTlsUfdNegativeQuittungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		return super.Fg255Typ016Id002AtgTlsGloNegativeQuittungAspTlsAntwort(deBlock, data, dataDescription);
	}

	/**
	 * FG 3 : Typ: 32 Id: 3 Name: UFD-Betriebsparameter (Abruf / Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 5 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1.. 254, 255 ,(evt. Clusterkanal)1 ]
	 * Byte 3           Typ der DE-Daten                        [ 32 ]
	 * Byte 4           Erfassungsperiodendauer                 [ 0..65535 s ] low Byte
	 * Byte 5           Erfassungsperiodendauer                 high Byte
	 * Byte 6           Übertragungsverfahren                   siehe unten
	 * 1) im Anwendungsfall Clusterkanal gilt entsprechend die Definition Seite A6.2.6 - 8, Kapitel 6.2.7
	 * </pre>
	 * Inhalt Byte Erfassungsperiodendauer<br> Der Wert "Erfassungsperiodendauer" gibt an, in welchen zeitlichen Abständen
	 * die Übertragung des Messergebnisses erfolgen soll. Es sind nur Werte zugelassen, die ein ganzzahliges Vielfaches
	 * haben, welches 24 Stunden ergibt (1, 2, 3, 4, 5, 6, 10, 12, 15, 20, und 30 Minuten und 1, 1½, 2, 3, 4, 6, 8 und 12
	 * Stunden).<br> Davon zu unterscheiden ist die Dauer der Messperiode. Die Messperiode ist sensor- und
	 * herstellerspezifisch. Beispielsweise liegt dem Mittelwert für die Windgeschwindigkeit eine bestimmte Messperiode
	 * zugrunde.<br> Der Wert "Erfassungsperiodendauer" ist nur relevant bei der zyklischen Abgabe von Meldungen (Byte
	 * "Übertragungsverfahren" = 1).<br> <br> Inhalt Byte Übertragungsverfahren<br> Das Byte gibt Anweisung, wann die Daten
	 * der Messwertgeber zur Zentrale übertragen werden: Entweder auf Abruf durch die Zentrale (Ereignisklasse 2), spontan
	 * nach Ende der o.g. Erfassungsperiode (Ereignisklasse 1). Meldung nur nach Abruf durch die Zentrale bedeutet, dass
	 * der DE ohne Aufforderung keine Meldungen mehr abgibt.
	 * <pre>
	 *         0: Meldung nur nach Abruf
	 *         1: Zyklische Abgabe von Meldungen
	 *    2..255: nicht definiert
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Erfassungsperiodendauer_-Wert aus Telegramm ------------------------------------------------------------------
		int _Erfassungsperiodendauer_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();

		switch (_Erfassungsperiodendauer_) {
			case 0:// Fehler HB, sendet immer 0
				_Erfassungsperiodendauer_ = 60;
			case 60:// 1 Minute
			case 120:// 2 Minuten
			case 180:// 3 Minuten
			case 240:// 4 Minuten
			case 300:// 5 Minuten
			case 360:// 6 Minuten
			case 600:// 10 Minuten
			case 720:// 12 Minuten
			case 900:// 15 Minuten
			case 1200:// 20 Minuten
			case 1800:// 30 Minuten
			case 3600:// 1 Stunde
			case 5400:// 1½ Stunden
			case 7200:// 2 Stunden
			case 10800:// 3 Stunden
			case 14400:// 4 Stunden
			case 21600:// 6 Stunden
			case 28800:// 8 Stunden
			case 43200:// 12 Stunden
				data.getUnscaledValue("Erfassungsperiodendauer").set(_Erfassungsperiodendauer_);
				break;
			default:
				debug.warning("Erfassungsperiodendauer ungültig : " + _Erfassungsperiodendauer_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());
				return resultData;
		}

		// Übertragungsverfahren-Byte aus Telegramm --------------------------------------------------------------------
		int _Übertragungsverfahren_ = deBlock.getDeData().readUnsignedByteAsInt();

		switch (_Übertragungsverfahren_) {
			case 0:
			case 1:
				data.getUnscaledValue("Übertragungsverfahren").set(_Übertragungsverfahren_);
				break;
			default:
				debug.warning("Übertragungsverfahren ungültig : " + _Übertragungsverfahren_ + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());
				return resultData;
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 32 Id: 3 Name: UFD-Betriebsparameter (Abruf / Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Position         Bezeichnung                             Erläuterung
	 * Byte 1           Länge DE-Block                          Länge des folgenden DE-Blocks [ 5 ]
	 * Byte 2           Daten-Endgeräte-Kanal                   [ 1.. 254, 255 ,(evt. Clusterkanal)1 ]
	 * Byte 3           Typ der DE-Daten                        [ 32 ]
	 * Byte 4           Erfassungsperiodendauer                 [ 0..65535 s ] low Byte
	 * Byte 5           Erfassungsperiodendauer                 high Byte
	 * Byte 6           Übertragungsverfahren                   siehe unten
	 * 1) im Anwendungsfall Clusterkanal gilt entsprechend die Definition Seite A6.2.6 - 8, Kapitel 6.2.7
	 * </pre>
	 * Inhalt Byte Erfassungsperiodendauer<br> Der Wert "Erfassungsperiodendauer" gibt an, in welchen zeitlichen Abständen
	 * die Übertragung des Messergebnisses erfolgen soll. Es sind nur Werte zugelassen, die ein ganzzahliges Vielfaches
	 * haben, welches 24 Stunden ergibt (1, 2, 3, 4, 5, 6, 10, 12, 15, 20, und 30 Minuten und 1, 1½, 2, 3, 4, 6, 8 und 12
	 * Stunden).<br> Davon zu unterscheiden ist die Dauer der Messperiode. Die Messperiode ist sensor- und
	 * herstellerspezifisch. Beispielsweise liegt dem Mittelwert für die Windgeschwindigkeit eine bestimmte Messperiode
	 * zugrunde.<br> Der Wert "Erfassungsperiodendauer" ist nur relevant bei der zyklischen Abgabe von Meldungen (Byte
	 * "Übertragungsverfahren" = 1).<br> <br> Inhalt Byte Übertragungsverfahren<br> Das Byte gibt Anweisung, wann die Daten
	 * der Messwertgeber zur Zentrale übertragen werden: Entweder auf Abruf durch die Zentrale (Ereignisklasse 2), spontan
	 * nach Ende der o.g. Erfassungsperiode (Ereignisklasse 1). Meldung nur nach Abruf durch die Zentrale bedeutet, dass
	 * der DE ohne Aufforderung keine Meldungen mehr abgibt.
	 * <pre>
	 *         0: Meldung nur nach Abruf
	 *         1: Zyklische Abgabe von Meldungen
	 *    2..255: nicht definiert
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg003Typ032Id003AtgTlsUfdBetriebsParameterAspParameterSoll(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Erfassungsperiodendauer_;
		int _Übertragungsverfahren_;

		// Umsetzung
		Data data = result.getData();// Datensatz holen

		_Erfassungsperiodendauer_ = data.getUnscaledValue("Erfassungsperiodendauer").intValue();
		_Übertragungsverfahren_ = data.getUnscaledValue("Übertragungsverfahren").intValue();

		// DeBlock als TlsData aufbauen...

		TlsData deApplicationData = new TlsData(3);
		deApplicationData.writeIntAsLowHighByte(_Erfassungsperiodendauer_);
		deApplicationData.writeIntAsByte(_Übertragungsverfahren_);

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 32, 3, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG 3 : Typ: 35 Id: 4 Name: UFD-xyz (Antwort) TLS-Datenbeschreibung:
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ035Id004AtgTlsUfdErgebnisMeldungTyp35AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Erfassungsperiodendauer_-Wert aus Telegramm ------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		//COM: data.getUnscaledValue("WERT").set(_Messwert_);

		debug.warning("Der Typ 35 in der FG 3 wird momentan nicht unterstützt." + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 48 Id: 4 Name: UFD-Lufttemperatur LT (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 -30...+60°C               0,1°C                         60,0 = 600d = 0258 h
	 *                                                                                0,0 = 0d = 0000 h
	 *                                                                                -0,1 = -1d = FFFF h
	 *                                                                                -30,0 = -300d = FED4 h
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * - 30 ... + 60 °C       Temperatur, der in Nähe der Fahrbahnmessstelle befindlichen Umgebungsluft.
	 *                        Messung muss weitgehend unbeeinflusst von direkter Sonneneinstrahlung und
	 *                        Windeinwirkung sein.
	 *                        Die Lufttemperatur wird in Grad Celsius angegeben.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ048Id004AtgTlsUfdErgebnisMeldungLuftTemperaturLTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_-Wert aus Telegramm -------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("Lufttemperatur").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ048Id004AtgTlsUfdErgebnisMeldungLuftTemperaturLTAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Lufttemperatur", 48);
	}

	/**
	 * FG 3 : Typ: 48 Id: 4 Name: UFD-Lufttemperatur LT (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 -30...+60°C               0,1°C                         60,0 = 600d = 0258 h
	 *                                                                                0,0 = 0d = 0000 h
	 *                                                                                -0,1 = -1d = FFFF h
	 *                                                                                -30,0 = -300d = FED4 h
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * - 30 ... + 60 °C       Temperatur, der in Nähe der Fahrbahnmessstelle befindlichen Umgebungsluft.
	 *                        Messung muss weitgehend unbeeinflusst von direkter Sonneneinstrahlung und
	 *                        Windeinwirkung sein.
	 *                        Die Lufttemperatur wird in Grad Celsius angegeben.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ048Id004AtgUfdsBoschungTemperaturLuftAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readSignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "Temperatur", 1, 32767.0f, 32639, 60);
	}

	/**
	 * FG 3 : Typ: 49 Id: 4 Name: UFD-xFahrbahnoberflächentemperatur FB (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 -30...+80°C               0,1°C                         80,0 = 800d = 0320 h
	 *                                                                                0,0 = 0d = 0000 h
	 *                                                                                -0,1 = -1d = FFFF h
	 *                                                                                -30,0 = -300d = FED4 h
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * - 30 ... + 80 °C       Temperatur der Oberfläche der Fahrbahn in Grad Celsius.
	 *                        Werden andere Zustände an der Oberfläche (z.B. Benetzung) erfasst, so muss die
	 *                        Oberflächentemperatur in örtlichen Bezug zu diesen Größen stehen.
	 *                        Die Fahrbahnoberflächentemperatur wird in Grad Celsius angegeben.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ049Id004AtgTlsUfdErgebnisMeldungFahrbahnOberFlächenTemperaturFBTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_-Wert aus Telegramm -------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("Fahrbahnoberflächentemperatur").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ049Id004AtgTlsUfdErgebnisMeldungFahrbahnOberFlächenTemperaturFBTAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Fahrbahnoberflächentemperatur", 49);
	}

	/**
	 * FG 3 : Typ: 49 Id: 4 Name: UFD-xFahrbahnoberflächentemperatur FB (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 -30...+80°C               0,1°C                         80,0 = 800d = 0320 h
	 *                                                                                0,0 = 0d = 0000 h
	 *                                                                                -0,1 = -1d = FFFF h
	 *                                                                                -30,0 = -300d = FED4 h
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * - 30 ... + 80 °C       Temperatur der Oberfläche der Fahrbahn in Grad Celsius.
	 *                        Werden andere Zustände an der Oberfläche (z.B. Benetzung) erfasst, so muss die
	 *                        Oberflächentemperatur in örtlichen Bezug zu diesen Größen stehen.
	 *                        Die Fahrbahnoberflächentemperatur wird in Grad Celsius angegeben.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ049Id004AtgUfdsBoschungTemperaturBodenAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readSignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "Temperatur", 1, 32767.0f, 32639, 60);
	}

	/**
	 * FG 3 : Typ: 50 Id: 4 Name: UFD-Fahrbahnfeuchte (Antwort) TLS-Datenbeschreibung:<br> DE-Typ 50 nicht mehr für
	 * Neuanwendung verwenden. Der DE-Typ wird durch Wasserfilmdicke(DE-Typ 72) ersetzt.
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ050Id004AtgTlsUfdErgebnisMeldungatgFahrbahnFeuchteFBFAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Fahrbahnfeuchte_ = deBlock.getDeData().readUnsignedByteAsInt();
		if ((_Fahrbahnfeuchte_ < 0) || (_Fahrbahnfeuchte_ > 100)) {
			debug.warning("Wert für Fahrbahnfeuchte ausserhalb zulässiger Grenzwerte (0-100%)." + Debug.NEWLINE + "De-Block (Osi7Adresse-DeNummer-FG-Typ-Id): " + deBlock.getLongKey() + " verworfen:" + Debug.NEWLINE + deBlock.toString());
		}
		else {
			data.getUnscaledValue("Fahrbahnfeuchte").set(_Fahrbahnfeuchte_);

			//Ergebnisdatensatz erzeugen...
			resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		}
		return resultData;
	}

	/**
	 * FG 3 : Typ: 51 Id: 4 Name: UFD-Zustand der Fahrbahnoberfläche FBZ (Antwort) TLS-Datenbeschreibung:<br> DE-Typ 51
	 * nicht mehr für Neuanwendung verwenden. Der DE-Typ wird durch eine neue Definition (DE-Typ 70) ersetzt.<br>
	 * <pre>
	 * Bit 0:   0 = Fahrbahn trocken
	 *          1 = Fahrbahn naß
	 * Bit 1:   0 = Fahrbahn nicht schneebedeckt
	 *          1 = Fahrbahn schneebedeckt
	 * Bit 2,3: 0 = Fahrbahn trocken (Bit 0 = 0)
	 *          1 = Feuchte 1
	 *          2 = Feuchte 2
	 *          3 = Feuchte 3
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ051Id004AtgTlsUfdErgebnisMeldungFahrbahnOberFlächeAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Fahrbahnoberfläche").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 52 Id: 4 Name: UFD-Restsalz RS (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 Bit                  0%...100%                 1% Lösung                     0% = 0 d = 00 h
	 *                                                                                100% = 100 d = 64 h
	 *                                                                                255 = 255 d = FFh
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 100 %            Sättigungsgrad des gelösten Salzes auf der Fahrbahn (bzw. auf der
	 *                        Fahrbahnsensorik). SR wird in Prozent (%) angegeben.
	 *                        0 % bedeutet kein gelöstes Salz (keine Gefrierpunkt-Erniedrigung)
	 *                        100 % bedeutet vollständig gelöstes Salz, die wässrige Lösung kann kein Salz mehr
	 *                        auflösen (Gefrierpunkt-Erniedrigung ist maximal).
	 *                        Es wird der IST-Wert übertragen.
	 * 255                    Sensorik kann auf Grund der herrschenden Bedingungen das Salzrest nicht
	 *                        bestimmen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ052Id004AtgTlsUfdErgebnisMeldungRestSalzRSAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Restsalz").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ052Id004AtgTlsUfdErgebnisMeldungRestSalzRSAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Restsalz", 52);
	}

	/**
	 * FG 3 : Typ: 52 Id: 4 Name: UFD-Restsalz RS (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 Bit                  0%...100%                 1% Lösung                     0% = 0 d = 00 h
	 *                                                                                100% = 100 d = 64 h
	 *                                                                                255 = 255 d = FFh
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 100 %            Sättigungsgrad des gelösten Salzes auf der Fahrbahn (bzw. auf der
	 *                        Fahrbahnsensorik). SR wird in Prozent (%) angegeben.
	 *                        0 % bedeutet kein gelöstes Salz (keine Gefrierpunkt-Erniedrigung)
	 *                        100 % bedeutet vollständig gelöstes Salz, die wässrige Lösung kann kein Salz mehr
	 *                        auflösen (Gefrierpunkt-Erniedrigung ist maximal).
	 *                        Es wird der IST-Wert übertragen.
	 * 255                    Sensorik kann auf Grund der herrschenden Bedingungen das Salzrest nicht
	 *                        bestimmen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ052Id004AtgUfdsBoschungRestSalzGehaltAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "RestSalzGehalt", 10, 32767.0f, 255, 60);
	}

	/**
	 * FG 3 : Typ: 53 Id: 4 Name: UFD-Niederschlagsintensität NI (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0... 200 mm/h             0,1 mm/h                      0,0 0d =0000h
	 *                                                                                200 2000d =07D0h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 200 mm/h         Fallender Niederschlag aller Art in mm/h. Die Intensität wird in Bezug auf
	 *                        Niederschlag in flüssiger Form durch Volumen (1 l/m2 = 1 mm) pro Zeiteinheit
	 *                        angegeben. Ist die Sensorik in der Lage, Niederschlag in gefrorener Form zu
	 *                        detektieren, wird das Wasseräquivalent (Menge nach Schmelzen) verwendet.
	 *                        Es wird der gleitende Mittelwert übertragen.
	 * </pre>
	 * Die VDI-Richtlinie VDI 3786 ist für Niederschlagsmessungen im Bereich Streckenstationen an Straßen nicht anwendbar,
	 * da die in dieser Richtlinie beschriebene Messtechnik in der Regel im Straßenbereich nicht anwendbar ist.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ053Id004AtgTlsUfdErgebnisMeldungNiederschlagsIntensitätNIAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("Niederschlagsintensität").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ053Id004AtgTlsUfdErgebnisMeldungNiederschlagsIntensitätNIAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Niederschlagsintensität", 53);
	}

	/**
	 * FG 3 : Typ: 53 Id: 4 Name: UFD-Niederschlagsintensität NI (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0... 200 mm/h             0,1 mm/h                      0,0 0d =0000h
	 *                                                                                200 2000d =07D0h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 200 mm/h         Fallender Niederschlag aller Art in mm/h. Die Intensität wird in Bezug auf
	 *                        Niederschlag in flüssiger Form durch Volumen (1 l/m2 = 1 mm) pro Zeiteinheit
	 *                        angegeben. Ist die Sensorik in der Lage, Niederschlag in gefrorener Form zu
	 *                        detektieren, wird das Wasseräquivalent (Menge nach Schmelzen) verwendet.
	 *                        Es wird der gleitende Mittelwert übertragen.
	 * </pre>
	 * Die VDI-Richtlinie VDI 3786 ist für Niederschlagsmessungen im Bereich Streckenstationen an Straßen nicht anwendbar,
	 * da die in dieser Richtlinie beschriebene Messtechnik in der Regel im Straßenbereich nicht anwendbar ist.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ053Id004AtgUfdsBoschungNiederschlagsIntensitätAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "NiederschlagsIntensität", 1, 32767.0f, 65535, 60);
	}

	/**
	 * FG 3 : Typ: 54 Id: 4 Name: UFD-Luftdruck LD (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 800...1200 hPa            1 hPa                         800 = 800 d = 0320 h
	 *                                                                                1200 = 1200 d = 04B0 h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 800 ... 1200 hPa       Atmosphärischer Luftdruck, bezogen auf Meereshöhe und Normbedingung (QNH),
	 *                        am Ort des Messquerschnittes in hPa.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ054Id004AtgTlsUfdErgebnisMeldungLuftDruckLDAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("Luftdruck").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ054Id004AtgTlsUfdErgebnisMeldungLuftDruckLDAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Luftdruck", 54);
	}

	/**
	 * FG 3 : Typ: 54 Id: 4 Name: UFD-Luftdruck LD (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 800...1200 hPa            1 hPa                         800 = 800 d = 0320 h
	 *                                                                                1200 = 1200 d = 04B0 h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 800 ... 1200 hPa       Atmosphärischer Luftdruck, bezogen auf Meereshöhe und Normbedingung (QNH),
	 *                        am Ort des Messquerschnittes in hPa.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ054Id004AtgUfdsBoschungLuftDruckAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "LuftDruck", 1, 32767.0f, 65535, 60);
	}

	/**
	 * FG 3 : Typ: 55 Id: 4 Name: UFD-Relative Luftfeuchte RLF (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 Bit                  10%...100%                1% rF                         10% = 10 d = 0A h
	 *                                                                                100% = 100 d = 64 h
	 *                                                                                1200 = 1200 d = 04B0 h
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 10 ... 100 %           Relative Luftfeuchte der in Nähe der Fahrbahnmessstelle befindlichen
	 *                        Umgebungsluft. Messung muss weitgehend unbeeinflusst von direkter
	 *                        Sonneneinstrahlung und Windeinwirkung sein.
	 *                        Die Rel. Feuchte sollte immer an der gleichen Stelle gemessen werden wie die
	 *                        Lufttemperatur (DE-Typ 48).
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ055Id004AtgTlsUfdErgebnisMeldungRelativeLuftFeuchteRLFAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("RelativeLuftfeuchte").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ055Id004AtgTlsUfdErgebnisMeldungRelativeLuftFeuchteRLFAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "RelativeLuftfeuchte", 55);
	}

	/**
	 * FG 3 : Typ: 55 Id: 4 Name: UFD-Relative Luftfeuchte RLF (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 Bit                  10%...100%                1% rF                         10% = 10 d = 0A h
	 *                                                                                100% = 100 d = 64 h
	 *                                                                                1200 = 1200 d = 04B0 h
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 10 ... 100 %           Relative Luftfeuchte der in Nähe der Fahrbahnmessstelle befindlichen
	 *                        Umgebungsluft. Messung muss weitgehend unbeeinflusst von direkter
	 *                        Sonneneinstrahlung und Windeinwirkung sein.
	 *                        Die Rel. Feuchte sollte immer an der gleichen Stelle gemessen werden wie die
	 *                        Lufttemperatur (DE-Typ 48).
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ055Id004AtgUfdsBoschungRelativeFeuchteLuftAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "RelativeFeuchteLuft", 10, 32767.0f, 255, 60);
	}

	/**
	 * FG 3 : Typ: 56 Id: 4 Name: UFD-Windrichtung WR (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0...359°                  1°                            0° (N) = 0 d = 0000 h
	 *                                                                                90°(O) = 90 d = 005A h
	 *                                                                                180°(S) = 180 d = 00B4 h
	 *                                                                                270°(W) = 270 d = 010E h
	 *                                                                                359° = 359 d = 0167 h
	 *                                                                                65535 ...= 65535 d = FFFFh
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 359°             Richtung aus der der Wind weht in Grad der Kompassrose angegeben.
	 *                        Die Windrichtung wird als vektorieller Mittelwert (Windrichtung und -
	 *                        geschwindigkeit) über ein gleitendes Zeitintervall von 10 Minuten ermittelt.
	 *                        Gleichzeitig sollte auch die Windgeschwindigkeit an der gleichen Stelle gemessen
	 *                        werden.
	 * -1 od. 65535           Keine Windrichtung ermittelbar, da Wind zu schwach.
	 * (FFFFh)                (In der Regel bei Windgeschwindigkeiten &lt; 0,5 m/s)
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ056Id004AtgTlsUfdErgebnisMeldungWindRichtungWRAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		if ((0 <= _Messwert_ && _Messwert_ <= 359) || _Messwert_ == 65535) {
			data.getUnscaledValue("Windrichtung").set(_Messwert_);
		}
		else {
			debug.warning("Windrichtung ausserhalb des gütligen Bereichs (0-359): " + _Messwert_);
			data.getUnscaledValue("Windrichtung").set(65535);
		}
		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ056Id004AtgTlsUfdErgebnisMeldungWindRichtungWRAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Windrichtung", 56);
	}

	/**
	 * FG 3 : Typ: 56 Id: 4 Name: UFD-Windrichtung WR (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0...359°                  1°                            0° (N) = 0 d = 0000 h
	 *                                                                                90°(O) = 90 d = 005A h
	 *                                                                                180°(S) = 180 d = 00B4 h
	 *                                                                                270°(W) = 270 d = 010E h
	 *                                                                                359° = 359 d = 0167 h
	 *                                                                                65535 ...= 65535 d = FFFFh
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 359°             Richtung aus der der Wind weht in Grad der Kompassrose angegeben.
	 *                        Die Windrichtung wird als vektorieller Mittelwert (Windrichtung und -
	 *                        geschwindigkeit) über ein gleitendes Zeitintervall von 10 Minuten ermittelt.
	 *                        Gleichzeitig sollte auch die Windgeschwindigkeit an der gleichen Stelle gemessen
	 *                        werden.
	 * -1 od. 65535           Keine Windrichtung ermittelbar, da Wind zu schwach.
	 * (FFFFh)                (In der Regel bei Windgeschwindigkeiten &lt; 0,5 m/s)
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ056Id004AtgUfdsBoschungWindRichtungAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "WindRichtung", 10, 32767.0f, 65535, 60);
	}

	/**
	 * FG 3 : Typ: 57 Id: 4 Name: UFD-Windgeschwindigkeit (Mittel) WGM(Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0.0 ... 60,0 m/s          0,1 m/s                       0,0 = 0d = 0000h
	 *                                                                                60,0 = 600d = 0258h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 60,0 m/s         Geschwindigkeit des Windes in horizontaler Ebene (Komponente) in Meter pro
	 *                        Sekunde.
	 *                        Windgeschwindigkeit (Mittel) wird als vektorieller Mittelwert über ein gleitendes
	 *                        Zeitintervall von 10 Minuten ermittelt.
	 *                        Bei gleichzeitiger Messung der Windrichtung sind die Sensoren an der gleichen
	 *                        Stelle zu platzieren.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ057Id004AtgTlsUfdErgebnisMeldungWindGeschwindigkeitMittelWertWGMAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("WindgeschwindigkeitMittel").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ057Id004AtgTlsUfdErgebnisMeldungWindGeschwindigkeitMittelWertWGMAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "WindgeschwindigkeitMittel", 57);
	}

	/**
	 * FG 3 : Typ: 57 Id: 4 Name: UFD-Windgeschwindigkeit (Mittel) WGM(Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0.0 ... 60,0 m/s          0,1 m/s                       0,0 = 0d = 0000h
	 *                                                                                60,0 = 600d = 0258h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 60,0 m/s         Geschwindigkeit des Windes in horizontaler Ebene (Komponente) in Meter pro
	 *                        Sekunde.
	 *                        Windgeschwindigkeit (Mittel) wird als vektorieller Mittelwert über ein gleitendes
	 *                        Zeitintervall von 10 Minuten ermittelt.
	 *                        Bei gleichzeitiger Messung der Windrichtung sind die Sensoren an der gleichen
	 *                        Stelle zu platzieren.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ057Id004AtgUfdsBoschungWindGeschwindigkeitMittelWertAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "WindGeschwindigkeitMittel", 0.36f, 32767.0f, 65535, 60);
	}

	/**
	 * FG 3 : Typ: 58 Id: 4 Name: UFD-Schneehöhe(Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 Bit                  0 ... 200 cm              1 cm                          0 = 00d = 00h
	 *                                                                                200 = 200d = C8h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 200 cm           Gibt die Höhe der Schneebedeckung an einer definierten Messstelle in Bezug zu
	 *                        einer ebenen, senkrechten Fläche an.
	 *                        0 cm wird gemeldet, wenn die ebene senkrechte Fläche völlig unbedeckt ist. Die
	 *                        Schneeoberfläche sollte durch die Messung nicht beeinflusst werden.
	 *                        Die Schneehöhe hat nichts mit der Bedeckung der Straßenoberfläche von
	 *                        Fahrspuren zu tun.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ058Id004AtgTlsUfdErgebnisMeldungSchneeHöheSHAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Schneehöhe").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 59 Id: 4 Name: UFD-Fahrbahnglätte (Antwort) TLS-Datenbeschreibung: <br> Dieser DE-Typ wird in FG3 nicht
	 * definiert. Ferner bleibt es den Herstellern von Glättemeldeanlagen überlassen für Glättezustände und/oder
	 * Glättewarnungen herstellerspezifische DE-Typ (zwischen 128 und 254) zu definieren.<br> <br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ059Id004AtgTlsUfdErgebnisMeldungFahrbahnGlätteFBGAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Wert1_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _Wert2_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Wert1").set(_Wert1_);
		data.getUnscaledValue("Wert2").set(_Wert2_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 60 Id: 4 Name: UFD-Sichtweite SW (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 10 ... 1000 m             1 m                           10 = 10d = 000Ah
	 *                                                                                1000 = 1000d = 03E8h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 10 ... 1000 m          Sichtweite bezogen auf eine Kontrastschwelle von 5% an der Messstelle in Meter.
	 *                        Ist diejenige Entfernung, in der zwei aneinandergrenzende Flächen, deren
	 *                        Kontrastunterschied 5% beträgt, gerade noch unterschieden werden können.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 * Die absolute Genauigkeit auf die gesamte Sichtweite kann bis 20% betragen.<br> <br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ060Id004AtgTlsUfdErgebnisMeldungSichtWeiteSWAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("Sichtweite").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ060Id004AtgTlsUfdErgebnisMeldungSichtWeiteSWAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Sichtweite", 60);
	}

	/**
	 * FG 3 : Typ: 60 Id: 4 Name: UFD-Sichtweite SW (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 10 ... 1000 m             1 m                           10 = 10d = 000Ah
	 *                                                                                1000 = 1000d = 03E8h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 10 ... 1000 m          Sichtweite bezogen auf eine Kontrastschwelle von 5% an der Messstelle in Meter.
	 *                        Ist diejenige Entfernung, in der zwei aneinandergrenzende Flächen, deren
	 *                        Kontrastunterschied 5% beträgt, gerade noch unterschieden werden können.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 * Die absolute Genauigkeit auf die gesamte Sichtweite kann bis 20% betragen.<br> <br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ060Id004AtgUfdsBoschungSichtWeiteAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		// Messwertbeschränkung nach oben hin durchführen, da Werte über 65000 vom Sensor geliefert werden.
		if ((messWert != 65535) && (messWert > 10000)) messWert = 10000;
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "SichtWeite", 1, 32767.0f, 65535, 60);
	}

	/**
	 * FG 3 : Typ: 61 Id: 4 Name: UFD-Helligkeit HK (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0.. 60000 Lx              1 Lx                          0 = 0 d = 0000 h
	 *                                                                                60000 = 60000 d = EA60 h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 60000 Lx         Helligkeit ist definiert als Beleuchtungsstärke (Anteil des auf eine ebene Messfläche
	 *                        senkrecht auftreffenden Komponente des Lichtstroms im sichtbaren Spektralbereich)
	 *                        in Lux angegeben.
	 *                        Die Ausrichtung des Sensors sollte beachtet werden.
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ061Id004AtgTlsUfdErgebnisMeldungHelligkeitHKAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("Helligkeit").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 62 Id: 4 Name: UFD-Niederschlagsmenge (alt) (Antwort) TLS-Datenbeschreibung:<br> DE-Typ 62 nicht mehr
	 * für Neuanwendungen verwenden. Statt dessen Niederschlagsintensität (DE-Typ 53) anwenden.<br> <br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ062Id004AtgTlsUfdErgebnisMeldungNiederschlagsMengeNMAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Wert1_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _Wert2_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Wert1").set(_Wert1_);
		data.getUnscaledValue("Wert2").set(_Wert2_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 63 Id: 4 Name: UFD-Niederschlagsart (alt)(Antwort) TLS-Datenbeschreibung:<br> DE-Typ 63 nicht mehr für
	 * Neuanwendungen verwenden. Statt dessen DE-Typ 71 Niederschlagsart anwenden.<br> <br>
	 * <pre>
	 * Bit 0:   0 = Fahrbahn trocken
	 *          1 = Fahrbahn naß
	 * Bit 1:   0 = Fahrbahn nicht schneebedeckt
	 *          1 = Fahrbahn schneebedeckt
	 * Bit 2,3: 0 = Fahrbahn trocken (Bit 0 = 0)
	 *          1 = Regen 1
	 *          2 = Regen 2
	 *          3 = Regen 3
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ063Id004AtgTlsUfdErgebnisMeldungNiederschlagAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Niederschlag").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 64 Id: 4 Name: UFD-Windgeschwindigkeit (Spitze) WGS (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0.0 ... 60,0 m/s          0,1 m/s                       0,0 = 0d = 0000h
	 *                                                                                60,0 = 600d = 0258h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 60,0 m/s         Geschwindigkeit des Windes in horizontaler Ebene (Komponente) in Meter pro
	 *                        Sekunde.
	 *                        Windgeschwindigkeit (Spitze) ist die maximale Windgeschwindigkeit der letzten 10
	 *                        Minuten.
	 *                        Bei gleichzeitiger Messung der Windrichtung sind die Sensoren an der gleichen
	 *                        Stelle zu platzieren.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ064Id004AtgTlsUfdErgebnisMeldungWindGeschwindigkeitSpitzenWertWGSAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("WindgeschwindigkeitSpitze").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ064Id004AtgTlsUfdErgebnisMeldungWindGeschwindigkeitSpitzenWertWGSAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "WindgeschwindigkeitSpitze", 64);
	}

	/**
	 * FG 3 : Typ: 64 Id: 4 Name: UFD-Windgeschwindigkeit (Spitze) WGS (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 0.0 ... 60,0 m/s          0,1 m/s                       0,0 = 0d = 0000h
	 *                                                                                60,0 = 600d = 0258h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * 0 ... 60,0 m/s         Geschwindigkeit des Windes in horizontaler Ebene (Komponente) in Meter pro
	 *                        Sekunde.
	 *                        Windgeschwindigkeit (Spitze) ist die maximale Windgeschwindigkeit der letzten 10
	 *                        Minuten.
	 *                        Bei gleichzeitiger Messung der Windrichtung sind die Sensoren an der gleichen
	 *                        Stelle zu platzieren.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ064Id004AtgUfdsBoschungWindGeschwindigkeitSpitzenWertAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "WindGeschwindigkeitSpitze", 0.36f, 32767.0f, 65535, 60);
	}

	/**
	 * <pre>FG 3 : Typ: 65 Id: 4 Name: UFD-Gefriertemperatur GT(Antwort) TLS-Datenbeschreibung: </pre> <pre> Format
	 * Bereich                   Auflösung                     Codierung 16 Bit                 -30...0 °C 0,1°C 0,0 = 0d =
	 * 0000 h -0,1 = -1d = FFFF h -30,0 = -300d = FED4 h
	 *
	 * Inhalt bzw.            Definition Ausprägung
	 *
	 * - 30 ... - 0.1 °C      Die Gefriertemperatur ist diejenige Temperatur in Grad Celsius, bei der die an der Oberfläche
	 * der Sonde vorhandene wässrige Taumittel-Lösung gefrieren (Übergang in den festen Aggregatszustand) könnte. Es wird
	 * der IST-Wert übertragen. 0 °C                   Keine Gefriertemperatur-Erniedrigung bzw. keine
	 * Taumittelkonzentration messbar bzw. die Bedingungen lassen eine Bestimmung nicht zu. Wenn gleichzeitig auch
	 * Salzrest (DE-Typ 52) übertragen wird ist auch dieser Wert zu betrachten (255 = nicht messbar). </pre> Es sollte
	 * unbedingt beachtet werden, dass mit keinem Messverfahren (passiv oder aktiv) die Taumittelmenge auf der Oberfläche
	 * und damit die bei Lösung zu erwartende Gefrierpunktserniedrigung (die zudem von der Benetzung abhängig ist) bestimmt
	 * werden kann, wenn nicht eine ausreichende Lösung vorliegt. Daher ist im trockenen oder abtrocknenden Zustand (bei
	 * Benetzungen unter 50 ml/m2) in der Regel keine Messung möglich. Es sollte daher für den Salzrest 255 und für die
	 * Gefrierpunktserniedrigung 0 °C gemeldet werden.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ065Id004AtgTlsUfdErgebnisMeldungGefrierTemperaturGTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("Gefriertemperatur").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ065Id004AtgTlsUfdErgebnisMeldungGefrierTemperaturGTAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Gefriertemperatur", 65);
	}

	/**
	 * <pre>FG 3 : Typ: 65 Id: 4 Name: UFD-Gefriertemperatur GT(Antwort) TLS-Datenbeschreibung: </pre> <pre> Format
	 * Bereich                   Auflösung                     Codierung 16 Bit                 -30...0 °C 0,1°C 0,0 = 0d =
	 * 0000 h -0,1 = -1d = FFFF h -30,0 = -300d = FED4 h
	 *
	 * Inhalt bzw.            Definition Ausprägung
	 *
	 * - 30 ... - 0.1 °C      Die Gefriertemperatur ist diejenige Temperatur in Grad Celsius, bei der die an der Oberfläche
	 * der Sonde vorhandene wässrige Taumittel-Lösung gefrieren (Übergang in den festen Aggregatszustand) könnte. Es wird
	 * der IST-Wert übertragen. 0 °C                   Keine Gefriertemperatur-Erniedrigung bzw. keine
	 * Taumittelkonzentration messbar bzw. die Bedingungen lassen eine Bestimmung nicht zu. Wenn gleichzeitig auch
	 * Salzrest (DE-Typ 52) übertragen wird ist auch dieser Wert zu betrachten (255 = nicht messbar). </pre> Es sollte
	 * unbedingt beachtet werden, dass mit keinem Messverfahren (passiv oder aktiv) die Taumittelmenge auf der Oberfläche
	 * und damit die bei Lösung zu erwartende Gefrierpunktserniedrigung (die zudem von der Benetzung abhängig ist) bestimmt
	 * werden kann, wenn nicht eine ausreichende Lösung vorliegt. Daher ist im trockenen oder abtrocknenden Zustand (bei
	 * Benetzungen unter 50 ml/m2) in der Regel keine Messung möglich. Es sollte daher für den Salzrest 255 und für die
	 * Gefrierpunktserniedrigung 0 °C gemeldet werden.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ065Id004AtgUfdsBoschungTemperaturGefrierPunktAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readSignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "Temperatur", 1, 32767.0f, 32639, 60);
	}

	/**
	 * FG 3 : Typ: 66 Id: 4 Name: UFD-Taupunkttemperatur TPT (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 60 °C          0,1 °C                        60,0 = 600d = 0258h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                 -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * - 30 ... +60 °C        Taupunkttemperatur ist diejenige Temperatur, bei der die in der Umgebungsluft der
	 *                        Messstelle (am Ort der Temperatur und Feuchte-Sensorik) befindliche Feuchte zu
	 *                        kondensieren beginnt.
	 *                        Die Taupunkttemperatur ist aus der Temperatur und der relativen Feuchte der Luft
	 *                        zu bestimmen.
	 *                        Zur Bestimmung ist die in den neuesten betriebstechnischen Anforderungen für
	 *                        Glättemeldeanlagen enthaltene Berechnungsvorschrift zu verwenden. Dieser
	 *                        Berechnungsvorschrift liegen auch die im Wetterdienst verwendeten
	 *                        Taupunkttabellen zugrunde.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ066Id004AtgTlsUfdErgebnisMeldungTaupunktTemperaturTPTAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("Taupunkttemperatur").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ066Id004AtgTlsUfdErgebnisMeldungTaupunktTemperaturTPTAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Taupunkttemperatur", 66);
	}

	/**
	 * FG 3 : Typ: 66 Id: 4 Name: UFD-Taupunkttemperatur TPT (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 60 °C          0,1 °C                        60,0 = 600d = 0258h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                 -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 *
	 * - 30 ... +60 °C        Taupunkttemperatur ist diejenige Temperatur, bei der die in der Umgebungsluft der
	 *                        Messstelle (am Ort der Temperatur und Feuchte-Sensorik) befindliche Feuchte zu
	 *                        kondensieren beginnt.
	 *                        Die Taupunkttemperatur ist aus der Temperatur und der relativen Feuchte der Luft
	 *                        zu bestimmen.
	 *                        Zur Bestimmung ist die in den neuesten betriebstechnischen Anforderungen für
	 *                        Glättemeldeanlagen enthaltene Berechnungsvorschrift zu verwenden. Dieser
	 *                        Berechnungsvorschrift liegen auch die im Wetterdienst verwendeten
	 *                        Taupunkttabellen zugrunde.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ066Id004AtgUfdsBoschungTemperaturTauPunktAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readSignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "Temperatur", 1, 32767.0f, 32639, 60);
	}

	/**
	 * FG 3 : Typ: 67 Id: 4 Name: UFD-Bodentemperatur in Tiefe 1 TT1 (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 80 °C          0,1 °C                        80,0 = 800d = 0320h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * - 30 ... + 80 °C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
	 *                        Oberfläche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
	 *                        in der betreffenden Tiefe
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ067Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe1TT1AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("BodentemperaturInTiefe1").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ067Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe1TT1AspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "BodentemperaturInTiefe1", 67);
	}

	/**
	 * FG 3 : Typ: 67 Id: 4 Name: UFD-Bodentemperatur in Tiefe 1 TT1 (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 80 °C          0,1 °C                        80,0 = 800d = 0320h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * - 30 ... + 80 °C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
	 *                        Oberfläche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
	 *                        in der betreffenden Tiefe
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ067Id004AtgUfdsBoschungTemperaturBodenTiefe1AspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readSignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "Temperatur", 1, 32767.0f, 32639, 60);
	}

	/**
	 * FG 3 : Typ: 68 Id: 4 Name: UFD-UFD-Bodentemperatur in Tiefe 2 TT2 (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 80 °C          0,1 °C                        80,0 = 800d = 0320h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * - 30 ... + 80 °C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
	 *                        Oberfläche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
	 *                        in der betreffenden Tiefe
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ068Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe2TT2AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("BodentemperaturInTiefe2").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ068Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe2TT2AspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "BodentemperaturInTiefe2", 68);
	}

	/**
	 * FG 3 : Typ: 69 Id: 4 Name: UFD-UFD-Bodentemperatur in Tiefe 3 TT3 (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 80 °C          0,1 °C                        80,0 = 800d = 0320h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * - 30 ... + 80 °C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
	 *                        Oberfläche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
	 *                        in der betreffenden Tiefe
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 * Die Tiefe 1 liegt der Oberfläche immer am nächsten. Es gilt Tiefe 1 &lt; Tiefe 2 &lt; Tiefe 3. Tiefe 3 sollte immer 30 cm
	 * betragen. Wenn nur eine Bodentiefentemperatur gemessen wird, sollte immer als Tiefe 30 cm (DE-Typ 69) verwendet
	 * werden.<br> <br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ069Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe3TT3AspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readSignedLowHighByteAsInt();
		data.getUnscaledValue("BodentemperaturInTiefe3").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ069Id004AtgTlsUfdErgebnisMeldungTemperaturInTiefe3TT3AspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "BodentemperaturInTiefe3", 69);
	}

	/**
	 * FG 3 : Typ: 69 Id: 4 Name: UFD-UFD-Bodentemperatur in Tiefe 3 TT3 (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 Bit                 - 30 ... + 80 °C          0,1 °C                        80,0 = 800d = 0320h
	 *                                                                                 0,0 = 0d = 0000h
	 *                                                                                -0,1 = -1d = FFFFh
	 *                                                                                -30,0 = -300d = FED4h
	 *
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * - 30 ... + 80 °C       Temperatur gemessen in der Tiefe 1 unter der Fahrbahn. Ist eine Fahrbahnsonde in die
	 *                        Oberfläche eingelassen, so befindet sich die Messstelle direkt unter der Fahrbahnsonde
	 *                        in der betreffenden Tiefe
	 *                        Es wird der IST-Wert übertragen.
	 * </pre>
	 * Die Tiefe 1 liegt der Oberfläche immer am nächsten. Es gilt Tiefe 1 &lt; Tiefe 2 &lt; Tiefe 3. Tiefe 3 sollte immer 30 cm
	 * betragen. Wenn nur eine Bodentiefentemperatur gemessen wird, sollte immer als Tiefe 30 cm (DE-Typ 69) verwendet
	 * werden.<br> <br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ069Id004AtgUfdsBoschungTemperaturBodenTiefe3AspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readSignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "Temperatur", 1, 32767.0f, 32639, 60);
	}

	/**
	 * FG 3 : Typ: 70 Id: 4 Name: UFD-Zustand der Fahrbahnoberfläche FBZ (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0                      Fahrbahn ist vollkommen Trocken (&lt; ca. 30 ml/m2 = 0,03 mm), schnee- und eisfrei
	 * 1                      Fahrbahn ist feucht bzw. nass, oder schnee- oder eisbedeckt. Die Benetzung bzw.
	 *                        Bedeckung übersteigt ca. 30 ml/m2 = 0,03 mm.
	 *                        Nähere Differenzierung der Bedeckungsart nicht möglich.
	 * 2 ... 31               frei für Erweiterungen
	 * 32                     Fahrbahn ist benetzt mit flüssigem Wasser bzw. wässriger Lösung.
	 *                        Die Menge übersteigt ca. 30 ml/m2 = 0,03 mm.
	 *                        Nähere Differenzierung nicht möglich.
	 * 33 ... 63              frei für Erweiterungen
	 * 64                     Fahrbahn ist bedeckt mit gefrorenem Wasser bzw. wässriger Lösung in festem
	 *                        Zustand. Weitere Differenzierung nicht möglich.
	 * 65                     Fahrbahn ist bedeckt mit Schnee oder Schneematsch. Gemisch von flüssigem und
	 *                        gefrorenem Wasser bzw. wässriger Lösung.
	 * 66                     Fahrbahn ist bedeckt mit Eis (festes, gefrorenes Wasser bzw. gefrorene wässrige
	 *                        Lösung)
	 * 67                     Fahrbahn ist bedeckt mit Rauhreif. Aus der Luft sublimierte Eiskristalle ohne
	 *                        deckende Eisfläche. Die Taupunkttemperatur liegt nahe der Fahrbahnoberflächen-
	 *                        Temperatur und liegt unter der Gefriertemperatur.
	 * 68 ... 127             frei für Erweiterungen
	 * 128 ... 254            frei für hersteller- oder anwendungsspezifische Codes
	 * 255                    Sensorik kann auf Grund der herrschenden Bedingungen Zustand nicht bestimmen.
	 * </pre>
	 * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktförmig sein kann und daher die
	 * Ergebnisse entsprechend interpretiert werden müssen.<br> Die Benetzung bzw. Bedeckung wird immer bezogen auf eine
	 * glatte ebene Fläche. Die Einschätzung der Gefährlichkeit einer Benetzung oder Bedeckung in Bezug zur Beschaffenheit
	 * des Fahrbahnbelags (Rauhigkeit etc.) und den besonderen Bedingungen der betreffenden Straßenstrecke muss in der
	 * Zentrale vorgenommen werden.<br> Es ist nicht erforderlich, dass die Sensortechnik alle Zustände direkt als solches
	 * detektieren kann. Es genügt vielmehr, wenn die zur Bildung der Zustände beitragenden Einflüsse gemessen werden
	 * können, so dass der betreffende Zustand als wahrscheinlich angenommen werden kann.<br> Von einer Fahrbahnmessstelle
	 * müssen nicht alle Ausprägungen unterstützt werden.<br> Die Menge der Benetzung bzw. Bedeckung, soweit bestimmbar,
	 * wird durch die Wasserfilmdicke (mm bzw. l/m2 ) angegeben.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ070Id004AtgTlsUfdErgebnisMeldungZustandDerFahrbahnOberFlächeFBZAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("ZustandFahrbahnoberfläche").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ070Id004AtgTlsUfdErgebnisMeldungZustandDerFahrbahnOberFlächeFBZAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "ZustandFahrbahnoberfläche", 70);
	}

	/**
	 * FG 3 : Typ: 70 Id: 4 Name: UFD-Zustand der Fahrbahnoberfläche FBZ (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0                      Fahrbahn ist vollkommen Trocken (&lt; ca. 30 ml/m2 = 0,03 mm), schnee- und eisfrei
	 * 1                      Fahrbahn ist feucht bzw. nass, oder schnee- oder eisbedeckt. Die Benetzung bzw.
	 *                        Bedeckung übersteigt ca. 30 ml/m2 = 0,03 mm.
	 *                        Nähere Differenzierung der Bedeckungsart nicht möglich.
	 * 2 ... 31               frei für Erweiterungen
	 * 32                     Fahrbahn ist benetzt mit flüssigem Wasser bzw. wässriger Lösung.
	 *                        Die Menge übersteigt ca. 30 ml/m2 = 0,03 mm.
	 *                        Nähere Differenzierung nicht möglich.
	 * 33 ... 63              frei für Erweiterungen
	 * 64                     Fahrbahn ist bedeckt mit gefrorenem Wasser bzw. wässriger Lösung in festem
	 *                        Zustand. Weitere Differenzierung nicht möglich.
	 * 65                     Fahrbahn ist bedeckt mit Schnee oder Schneematsch. Gemisch von flüssigem und
	 *                        gefrorenem Wasser bzw. wässriger Lösung.
	 * 66                     Fahrbahn ist bedeckt mit Eis (festes, gefrorenes Wasser bzw. gefrorene wässrige
	 *                        Lösung)
	 * 67                     Fahrbahn ist bedeckt mit Rauhreif. Aus der Luft sublimierte Eiskristalle ohne
	 *                        deckende Eisfläche. Die Taupunkttemperatur liegt nahe der Fahrbahnoberflächen-
	 *                        Temperatur und liegt unter der Gefriertemperatur.
	 * 68 ... 127             frei für Erweiterungen
	 * 128 ... 254            frei für hersteller- oder anwendungsspezifische Codes
	 * 255                    Sensorik kann auf Grund der herrschenden Bedingungen Zustand nicht bestimmen.
	 * </pre>
	 * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktförmig sein kann und daher die
	 * Ergebnisse entsprechend interpretiert werden müssen.<br> Die Benetzung bzw. Bedeckung wird immer bezogen auf eine
	 * glatte ebene Fläche. Die Einschätzung der Gefährlichkeit einer Benetzung oder Bedeckung in Bezug zur Beschaffenheit
	 * des Fahrbahnbelags (Rauhigkeit etc.) und den besonderen Bedingungen der betreffenden Straßenstrecke muss in der
	 * Zentrale vorgenommen werden.<br> Es ist nicht erforderlich, dass die Sensortechnik alle Zustände direkt als solches
	 * detektieren kann. Es genügt vielmehr, wenn die zur Bildung der Zustände beitragenden Einflüsse gemessen werden
	 * können, so dass der betreffende Zustand als wahrscheinlich angenommen werden kann.<br> Von einer Fahrbahnmessstelle
	 * müssen nicht alle Ausprägungen unterstützt werden.<br> Die Menge der Benetzung bzw. Bedeckung, soweit bestimmbar,
	 * wird durch die Wasserfilmdicke (mm bzw. l/m2 ) angegeben.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ070Id004AtgUfdsBoschungStraßenZustandAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedByteAsInt();

		// To
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "StraßenZustand", 1, 32767.0f, 255, 60);
	}

	/**
	 * FG 3 : Typ: 71 Id: 4 Name: UFD-Niederschlagsart (NS)(Antwort) TLS-Datenbesc hreibung:
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * alle                   In der Atmosphäre fallender Niederschlag.
	 *                        Wird gleichzeitig Niederschlagsintensität ermittelt, ist diese an der gleichen Stelle
	 *                        vorzunehmen.
	 *                        Es werden die Klassifikation und die Codes nach WMO Tabelle 4680 verwendet.
	 * 0                      kein Niederschlag
	 * 1 ... 39               nicht benutzen
	 * 40                     Niederschlag aller Art
	 *                        Nicht näher Klassifizier- und quantifizierbar oder Sensorik nicht dafür ausgelegt
	 * 41                     Leichter oder mittlerer Niederschlag aller Art ( &lt; 50 Partikel/Minute)
	 * 42                     Starker Niederschlag aller Art ( &gt; 50 Partikel/Minute)
	 * 43 ... 49              frei für Erweiterungen
	 * 50                     Sprühregen
	 *                        (keine weitere Klassifikation von Sprühregen möglich)
	 * 51 ... 59              weitere Klassifikation von Sprühregen n. WMO
	 * 60                     Regen bzw. flüssiger Niederschlag
	 *                        (Keine weitere Klassifikation von Regen möglich)
	 * 61 ... 69              Weitere Klassifikation von Regen n. WMO
	 * 70                     Schnee bzw. gefrorener Niederschlag
	 *                        (Keine weitere Klassifikation von gefrorenem Niederschlag möglich)
	 * 71 ... 73              Weiter Klassifikation von Schnee n. WMO
	 * 74 ... 76              Weitere Klassifikation von Graupel n. WMO
	 * 77 ... 79              Weitere Klassifikation von Hagel n. WMO
	 * 80 ... 127             frei für Erweiterungen
	 * 128 ... 254            frei für hersteller- und anwendungsspez. Erweiterungen
	 * 255                    Sensorik kann auf Grund der herrschenden Bedingungen Zustand nicht bestimmen
	 * </pre>
	 * Es wird darauf hingewiesen, dass von der Streckenstation bzw. der Sensorik nicht alle Ausprägungen unterstützt
	 * werden müssen. Welcher Differenzierungsgrad notwendig und sinnvoll ist, hängt von der Anwendung ab. Bei einfachen
	 * Anwendungen können die Ausprägungen 0 und 40, bei normalen Anforderungen 0, 60 und 70 ( entspricht den bisher
	 * verwendeten Ausprägungen 00, 01, 02 des Typs 63 ) ausreichen.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ071Id004AtgTlsUfdErgebnisMeldungNiederschlagsArtNSAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Niederschlagart").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 71 Id: 4 Name: UFD-Niederschlagsart (NS)(Antwort) TLS-Datenbesc hreibung:
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * alle                   In der Atmosphäre fallender Niederschlag.
	 *                        Wird gleichzeitig Niederschlagsintensität ermittelt, ist diese an der gleichen Stelle
	 *                        vorzunehmen.
	 *                        Es werden die Klassifikation und die Codes nach WMO Tabelle 4680 verwendet.
	 * 0                      kein Niederschlag
	 * 1 ... 39               nicht benutzen
	 * 40                     Niederschlag aller Art
	 *                        Nicht näher Klassifizier- und quantifizierbar oder Sensorik nicht dafür ausgelegt
	 * 41                     Leichter oder mittlerer Niederschlag aller Art ( &lt; 50 Partikel/Minute)
	 * 42                     Starker Niederschlag aller Art ( &gt; 50 Partikel/Minute)
	 * 43 ... 49              frei für Erweiterungen
	 * 50                     Sprühregen
	 *                        (keine weitere Klassifikation von Sprühregen möglich)
	 * 51 ... 59              weitere Klassifikation von Sprühregen n. WMO
	 * 60                     Regen bzw. flüssiger Niederschlag
	 *                        (Keine weitere Klassifikation von Regen möglich)
	 * 61 ... 69              Weitere Klassifikation von Regen n. WMO
	 * 70                     Schnee bzw. gefrorener Niederschlag
	 *                        (Keine weitere Klassifikation von gefrorenem Niederschlag möglich)
	 * 71 ... 73              Weiter Klassifikation von Schnee n. WMO
	 * 74 ... 76              Weitere Klassifikation von Graupel n. WMO
	 * 77 ... 79              Weitere Klassifikation von Hagel n. WMO
	 * 80 ... 127             frei für Erweiterungen
	 * 128 ... 254            frei für hersteller- und anwendungsspez. Erweiterungen
	 * 255                    Sensorik kann auf Grund der herrschenden Bedingungen Zustand nicht bestimmen
	 * </pre>
	 * Es wird darauf hingewiesen, dass von der Streckenstation bzw. der Sensorik nicht alle Ausprägungen unterstützt
	 * werden müssen. Welcher Differenzierungsgrad notwendig und sinnvoll ist, hängt von der Anwendung ab. Bei einfachen
	 * Anwendungen können die Ausprägungen 0 und 40, bei normalen Anforderungen 0, 60 und 70 ( entspricht den bisher
	 * verwendeten Ausprägungen 00, 01, 02 des Typs 63 ) ausreichen.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ071Id004AtgUfdsBoschungNiederschlagsArtAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedByteAsInt();

		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "NiederschlagsArt", 1, 32767.0f, 255, 60);
	}

	public DeBlock Fg003Typ071Id004AtgTlsUfdErgebnisMeldungNiederschlagsArtNSAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Niederschlagart", 71);

	}

	/**
	 * FG 3 : Typ: 72 Id: 4 Name: UFD-Wasserfilmdicke WFD (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 bit                 0,00...10,00 mm           0,01 mm                       0 = 0d = 0000h
	 *                                                                                10,00 = 1000d = 03E8h
	 *                                                                                65535 = 65535d = FFFFh
	 * </pre>
	 * Bei WFD oberhalb 0,1 mm wird nur eine Messgenauigkeit im 1/10 mm Bereich erreicht. Bei WFD oberhalb 2 mm liegt die
	 * Messgenauigkeit in der Regel im mm Bereich<br>
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0 ... 10,00 mm         Die rechnerische Wasserfilmdicke gibt die Fahrbahnbenetzung in mm (= Liter/m2 ) mit
	 *                        Wasser oder wässriger Lösung (Salzgemisch) an, welche sich am Messort bzw. auf der
	 *                        in der Fahrbahn eingelassenen Sonde befindet. Die Wasserfilmdicke ist immer bezogen
	 *                        auf eine glatte ebene Fläche.
	 * -1 od. 65535           Wasserfilmdicke auf Grund der herrschenden Bedingungen nicht bestimmbar oder feste
	 * (FFFFh)                bzw. gefrorene Bedeckung.
	 * </pre>
	 * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktförmig sein kann und daher die
	 * Ergebnisse entsprechend interpretiert werden müssen.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ072Id004AtgTlsUfdErgebnisMeldungWasserFilmDickeWFDAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		data.getUnscaledValue("Wasserfilmdicke").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	public DeBlock Fg003Typ072Id004AtgTlsUfdErgebnisMeldungWasserFilmDickeWFDAspTlsAntwort(ResultData result) {
		return tlsErgebnisDatenFG003TypXXXId004AnDeSenden(result, "Wasserfilmdicke", 72);
	}

	/**
	 * FG 3 : Typ: 72 Id: 4 Name: UFD-Wasserfilmdicke WFD (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 16 bit                 0,00...10,00 mm           0,01 mm                       0 = 0d = 0000h
	 *                                                                                10,00 = 1000d = 03E8h
	 *                                                                                65535 = 65535d = FFFFh
	 * </pre>
	 * Bei WFD oberhalb 0,1 mm wird nur eine Messgenauigkeit im 1/10 mm Bereich erreicht. Bei WFD oberhalb 2 mm liegt die
	 * Messgenauigkeit in der Regel im mm Bereich<br>
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0 ... 10,00 mm         Die rechnerische Wasserfilmdicke gibt die Fahrbahnbenetzung in mm (= Liter/m2 ) mit
	 *                        Wasser oder wässriger Lösung (Salzgemisch) an, welche sich am Messort bzw. auf der
	 *                        in der Fahrbahn eingelassenen Sonde befindet. Die Wasserfilmdicke ist immer bezogen
	 *                        auf eine glatte ebene Fläche.
	 * -1 od. 65535           Wasserfilmdicke auf Grund der herrschenden Bedingungen nicht bestimmbar oder feste
	 * (FFFFh)                bzw. gefrorene Bedeckung.
	 * </pre>
	 * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktförmig sein kann und daher die
	 * Ergebnisse entsprechend interpretiert werden müssen.<br>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ072Id004AtgUfdsBoschungWasserFilmDickeAspUfdsBoschungVorgabe(DeBlock deBlock, Data data, DataDescription dataDescription) {
		// MessWert-Wert aus Telegramm -------------------------------------------------------------------------------
		int messWert = deBlock.getDeData().readUnsignedLowHighByteAsInt();
		return tlsDatenNachResultUfdBoschung(messWert, data, dataDescription, "WasserFilmDicke", 10, 32767.0f, 65535, 60);
	}

	/**
	 * FG 3 : Typ: 132 Id: 4 Name: UFD-FahrBahn-Glättewarnung (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 bit                  0..254, 255                                             0 = 0d = 0000h
	 * </pre>
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0 ..254, 255           Bedeutung der Werte zur Zeit nicht bekannt.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ132Id004AtgTlsUfdErgebnisMeldungFahrbahnGlätteWarnungAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("MessWert").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 179 Id: 4 Name: UFD-FahrBahn-Zustand (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 bit                  0..254, 255                                             0 = 0d = 0000h
	 * </pre>
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0 ..254, 255           Bedeutung der Werte zur Zeit nicht bekannt.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ179Id004AtgTlsUfdErgebnisMeldungFahrbahnZustandAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("MessWert").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 3 : Typ: 132 Id: 4 Name: UFD-FahrBahn-InfrarotSensor (Antwort) TLS-Datenbeschreibung:
	 * <pre>
	 * Format                 Bereich                   Auflösung                     Codierung
	 * 8 bit                  0..254, 255                                             0 = 0d = 0000h
	 * </pre>
	 * <pre>
	 * Inhalt bzw.            Definition
	 * Ausprägung
	 * 0 ..254, 255           Bedeutung der Werte zur Zeit nicht bekannt.
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg003Typ230Id004AtgTlsUfdErgebnisMeldungFahrbahnInfrarotSensorAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
		int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("MessWert").set(_Messwert_);

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * Standardumsetzungsmethode, um ein empfangenen TLS-Ergebnisdatensatz der FG 3, Typ xxx, ID 4 in einen DE-Block zu
	 * konvertieren. Wird verwendet, um TLS-Daten über die VRZ-Leverkusen an die HB-UZen zu schicken.
	 *
	 * @param result Ergebnisdatensatz der entsprechenden Attributgruppe, der in einen DEBlock konvertiert werden soll.
	 * @param att    Der Name des Attributs (z. B. "Temperatur", der den eigentlichen Messwert enthält (DaV-seitig).
	 * @param typ    TLS-Typ des zu erzeugenden DE-Blocks.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	private DeBlock tlsErgebnisDatenFG003TypXXXId004AnDeSenden(ResultData result, String att, int typ) {
		final int id = 4;

		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int messWert;

		// Umsetzung
		Data data = result.getData();// Datensatz holen
		messWert = data.getUnscaledValue(att).intValue();

		// DeBlock als TlsData aufbauen...
		int anzBytes;

		switch (typ) {
			case 48:
			case 49:
			case 53:
			case 54:
			case 56:
			case 57:
			case 59:
			case 60:
			case 61:
			case 62:
			case 64:
			case 65:
			case 66:
			case 67:
			case 68:
			case 69:
			case 72:
				anzBytes = 2;
				break;
			default:
				anzBytes = 1;
		}

		TlsData deApplicationData;
		if (anzBytes == 2) {
			deApplicationData = new TlsData(anzBytes);
			deApplicationData.writeIntAsLowHighByte(messWert);
		}
		else {
			deApplicationData = new TlsData(anzBytes);
			deApplicationData.writeIntAsByte(messWert);
		}

		// DeBlock erzeugen
		try {
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), typ, id, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * Standardumsetzungsmethode, um #{@link de.bsvrz.kex.tls.osi7.DeBlock}-Daten in #{@link ResultData} eines
	 * Boschung-Sensors ("atg.ufdsBoschungXXX", "asp.ufdsBoschungVorgabe", "typ.ufdSensorBoschungXXX") zu konvertieren.
	 *
	 * @param messWert            Der Messwert des TLS-Sensors
	 * @param data                Das #{@link de.bsvrz.dav.daf.main.Data}-Objekt für die Zieldaten.
	 * @param dataDescription     Die #{@link DataDescription} für die Zieldaten.
	 * @param att                 Der Name des Attributs (z. B. "Temperatur", der den eigentlichen Messwert enthält
	 *                            (DaV-seitig).
	 * @param skalierung          Faktor, um den <code>float</code> Messwert aus den MeteoDaten unter Berücksichtigung der
	 *                            Skalierung für den Messwert in der Attributgruppe in eine UMSKALIERTE Ganzzahldarstellung
	 *                            umzurechnen. Beispiel: Format des Messwerts bei Boschung %6.1 (also mit einer
	 *                            Nachkommastelle), Skalierung des Attributs in der DaV-Attributgruppe 0,1 (also auch eine
	 *                            Nachkommastelle) ergibt einen Faktor von 10! Beispiel: Messwert (z.B. 32.1) wird mit
	 *                            Faktor 10 multipliziert, ergibt also 321. Wird dieser unskalierte Wert 321 in die Atg
	 *                            geschrieben, so erfolgt die Darstellung im DaV mit Berücksichtigung der Skalierung als
	 *                            32,1). Weitere Werte: Format %6.2 und Skalierung 0,01 ergibt Faktor 100, Format %6.1 und
	 *                            Skalierung 0,01 ergibt auch Faktor 100.
	 * @param nichtErmittelbar    Der Messwert, der von einem Boschung-Sensor für nicht ermittelbare Daten gesetzt wird, i.
	 *                            d. R. <code>32767.0f</code>.
	 * @param nichtErmittelbarTLS Der Wert in der TLS-ATG, der gesetzt wird, wenn der Boschung-Messwert nicht ermittelbar
	 *                            oder fehlerhaft ist.
	 * @param intervallDauer      Die Erfassungsintervalldauer in Sekunden des Boschungsensors. Bei einem negativen Wert
	 *                            wird automatisch die Standardvorgabe eingestellt (i. d. R. 60 Sekunden).
	 *
	 * @return Der Ergebnisdatensatz mit den konvertierten MeteoDaten.
	 */
	private ResultData tlsDatenNachResultUfdBoschung(int messWert, Data data, DataDescription dataDescription, String att, float skalierung, float nichtErmittelbar, int nichtErmittelbarTLS, int intervallDauer) {
		final int intervallDauerStandard = 60;// Standardintervalllänge 1 Minuten (60 Sekunden) bei TLS-Daten.
		if (intervallDauer < 0) {
			data.getTimeValue("T").setSeconds(intervallDauerStandard);
		}
		else {
			data.getTimeValue("T").setSeconds(intervallDauer);
		}

		// Messwert ermitteln, dazu Messwert skalieren und eventuelle Fehlerzustände berücksichtigen.
		if (messWert == nichtErmittelbarTLS) {
			//data.getUnscaledValue(att).set((int) (nichtErmittelbar * skalierung));
			data.getUnscaledValue(att).setText("nicht ermittelbar");
		}
		else {
			data.getUnscaledValue(att).set((int) (messWert * skalierung));
		}
		// Plausibilität setzen.
		data.getUnscaledValue("Plausibilität").set(100);
		// FehlerZustand setzen.
		data.getUnscaledValue("FehlerZustand").set(0);
		// Ergebnisdatensatz erzeugen, Zielobjekt wird durch aufrufende Methode ersetzt.
		return new ResultData(null, dataDescription, _eakConversionModul.getTimeStamp(), data);
	}

}
